/*
 * gtkjoy.c
 *
 * general routines for handling joystick events
 *
 * Copyright 1999 David Boynton.
 * http://trylinux.com/projects/joystick/
 */
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <linux/joystick.h>

#include "gtkjoy.h"

/* static */
static void joystick_event(gpointer data, gint fdsource, GdkInputCondition cond);

JoyData *   
joystick_new(char * joystick, void * owner, void * data)
{
	JoyData * joydata;

	joydata=(JoyData *)g_malloc(sizeof(JoyData));
	if (!joydata)
		return NULL;

	memset(joydata, 0, sizeof(JoyData));

	{ /* open joystick device */
		char *devname="/dev/js0";

		if ( joystick )
			devname=joystick;
		joydata->joystickfd= open(devname, O_RDONLY);

		if ( joydata->joystickfd < 0 ) {
			if ( errno == ENODEV ) {
				g_print("Device not configured - did you load the module?\n");
			} else {
				g_print("Couldn't open '%s'\n", devname);
				perror(devname);
			}
			return NULL;
		}
	}

	joydata->callback=(joystick_callback)owner; /* button callback */
	joydata->callback_data=(gpointer)data;

	{ /* load properties */
		char name[128] = "Unknown";

		fcntl(joydata->joystickfd, F_SETFL, O_NONBLOCK);
		ioctl(joydata->joystickfd, JSIOCGVERSION, &joydata->version);
		ioctl(joydata->joystickfd, JSIOCGAXES, &joydata->num_axis);
		ioctl(joydata->joystickfd, JSIOCGBUTTONS, &joydata->num_buttons);
		ioctl(joydata->joystickfd, JSIOCGNAME(128), name);
		joydata->device_name = strdup(name);
		joydata->gdktag=
				gdk_input_add(joydata->joystickfd, GDK_INPUT_READ, joystick_event, joydata);
	}

	return joydata;
}

void
joystick_close(JoyData * joydata)
{
	if ( joydata->gdktag ) 
		gdk_input_remove(joydata->gdktag);
	if ( joydata->joystickfd != -1 ) 
		close(joydata->joystickfd);
	if ( joydata->device_name ) 
		g_free(joydata->device_name);
	g_free(joydata);
}

static void 
joystick_event(gpointer data, gint fdsource, GdkInputCondition cond)
{
	struct js_event js;
	int readrc;
	JoyData * joydata;

	joydata=(gpointer)data;
	if (!joydata) {
		g_print("joystick_event: improper callback registration, %u\n", __LINE__);
		return;
	}

	readrc=read(fdsource, &js, sizeof(struct js_event));
	if (readrc == sizeof(struct js_event)) {
		switch ((js.type&3)) {
			case JS_EVENT_BUTTON:
				if ( js.number < JOYSTICK_MAXB ) 
					joydata->buttons[js.number]=js.value;
				break;
			case JS_EVENT_AXIS:
				if ( js.number < JOYSTICK_MAXA ) 
					joydata->axis[js.number]=js.value;
				break;
		}
		if ( joydata->callback )
			joydata->callback(js.type, js.number, joydata->callback_data);
	} else { 
		g_print("source(%d), %d bytes read\n", fdsource, readrc);
	}
}

